<?php

namespace Lc\CaracoleBundle\Model\Order;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Gedmo\Mapping\Annotation as Gedmo;
use Doctrine\ORM\Mapping as ORM;
use Lc\CaracoleBundle\Model\File\DocumentInterface;
use Lc\CaracoleBundle\Doctrine\Extension\FilterMerchantInterface;
use Lc\CaracoleBundle\Services\Order\OrderUtils;
use Lc\CaracoleBundle\Model\OrderStatus ;

/**
 * @ORM\MappedSuperclass()
 */
abstract class OrderShopModel extends AbstractEntity implements FilterMerchantInterface
{
        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\Merchant\MerchantInterface", inversedBy="productFamilies")
         * @ORM\JoinColumn(nullable=false)
         */
        protected $merchant;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\User\UserInterface", inversedBy="orders", fetch="EAGER")
         */
        protected $user;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\User\VisitorInterface", inversedBy="orders")
         */
        protected $visitor;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\Address\AddressInterface")
         */
        protected $invoiceAddress;

        /**
         * @ORM\Column(type="text", nullable=true)
         */
        protected $invoiceAddressText;

        /**
         * @ORM\Column(type="datetime", nullable=true)
         */
        protected $validationDate;

        /**
         * @ORM\Column(type="text", nullable=true)
         */
        protected $comment;

        /**
         * @ORM\Column(type="boolean", nullable=true)
         */
        protected $autoPayment;

        /**
         * @ORM\Column(type="string", length=31, nullable=true)
         */
        protected $meanPayment;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderStatusHistoryInterface", mappedBy="orderShop", orphanRemoval=true)
         */
        protected $orderStatusHistories;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderPaymentInterface", mappedBy="orderShop", orphanRemoval=true)
         */
        protected $orderPayments;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderProductInterface", mappedBy="orderShop", cascade={"persist", "remove"}, orphanRemoval=true, fetch="EAGER")
         */
        protected $orderProducts;

        /**
         * @ORM\Column(type="text", nullable=true)
         */
        protected $deliveryInfos;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\Order\OrderStatusInterface")
         * @ORM\JoinColumn(nullable=false)
         */
        protected $orderStatus;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderReductionCartInterface", mappedBy="orderShop", orphanRemoval=true)
         */
        protected $orderReductionCarts;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Order\OrderReductionCreditInterface", mappedBy="orderShop", orphanRemoval=true)
         */
        protected $orderReductionCredits;

        /**
         * @ORM\ManyToMany(targetEntity="Lc\CaracoleBundle\Model\Order\DocumentInterface", inversedBy="orderShops")
         */
        protected $documents;

        /**
         * @Gedmo\Blameable(on="create")
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\User\UserInterface")
         * @ORM\JoinColumn(nullable=true)
         */
        protected $createdBy;

        /**
         * @Gedmo\Blameable(on="update")
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\User\UserInterface")
         * @ORM\JoinColumn(nullable=true)
         */
        protected $updatedBy;

        /**
         * @ORM\OneToMany(targetEntity="Lc\CaracoleBundle\Model\Ticket\TicketInterface", mappedBy="orderShop")
         */
        protected $tickets;

        /**
         * @ORM\ManyToOne(targetEntity="Lc\CaracoleBundle\Model\Section\SectionInterface", inversedBy="orderShops")
         * @ORM\JoinColumn(nullable=false)
         */
        protected $section;

        public function __construct()
        {
                $this->orderStatusHistories = new ArrayCollection();
                $this->orderPayments = new ArrayCollection();
                $this->orderProducts = new ArrayCollection();
                $this->creditHistories = new ArrayCollection();
                $this->orderReductionCarts = new ArrayCollection();
                $this->orderReductionCredits = new ArrayCollection();
                $this->documents = new ArrayCollection();
        }

        public function __toString()
        {
                if($this->getValidationDate()) {
                        return 'Commande du '.$this->getValidationDate()->format('d/m/Y') ;
                }
                else {
                        return 'Commande #'.$this->getId() ;
                }
        }

        public function getValidationDate(): ?\DateTimeInterface
        {
                return $this->validationDate;
        }

        public function setValidationDate(\DateTimeInterface $validationDate): self
        {
                $this->validationDate = $validationDate;

                return $this;
        }

        public function getDateCreated()
        {
                $orderStatusHistory = $this->getOrderStatusHistory(OrderStatus::ALIAS_WAITING_DELIVERY);
                if ($orderStatusHistory) {
                        return $orderStatusHistory->getCreatedAt();
                }

                return null;
        }

        public function getOrderStatusHistory($status)
        {
                $orderStatusHistories = $this->getOrderStatusHistories();
                if (count($orderStatusHistories) > 0) {
                        foreach ($orderStatusHistories as $orderStatusHistory) {
                                if ($orderStatusHistory->getOrderStatus() == $status) {
                                        return $orderStatusHistory;
                                }
                        }
                }

                return null;
        }

        public function getMerchant(): ?Merchant
        {
                return $this->merchant;
        }

        public function setMerchant(?Merchant $merchant): self
        {
                $this->merchant = $merchant;

                return $this;
        }

        public function getUser(): ?User
        {
                return $this->user;
        }

        public function setUser(?User $user): self
        {
                $this->user = $user;

                return $this;
        }

        public function getInvoiceAddress(): ?Address
        {
                return $this->invoiceAddress;
        }

        public function setInvoiceAddress(?Address $invoiceAddress): self
        {
                $this->invoiceAddress = $invoiceAddress;

                return $this;
        }

        public function getInvoiceAddressText(): ?string
        {
                return $this->invoiceAddressText;
        }

        public function setInvoiceAddressText(string $invoiceAddressText): self
        {
                $this->invoiceAddressText = $invoiceAddressText;

                return $this;
        }

        public function getComment(): ?string
        {
                return $this->comment;
        }

        public function setComment(?string $comment): self
        {
                $this->comment = $comment;

                return $this;
        }

        public function getAutoPayment(): ?bool
        {
                return $this->autoPayment;
        }

        public function setAutoPayment(bool $autoPayment): self
        {
                $this->autoPayment = $autoPayment;

                return $this;
        }

        public function getMeanPayment(): ?string
        {
                return $this->meanPayment;
        }

        public function setMeanPayment(string $meanPayment): self
        {
                $this->meanPayment = $meanPayment;

                return $this;
        }

        /**
         * @return Collection|OrderStatusHistoryModel[]
         */
        public function getOrderStatusHistories(): Collection
        {
                return $this->orderStatusHistories;
        }

        public function addOrderStatusHistory(OrderStatusHistoryModel $orderStatusHistory): self
        {
                if (!$this->orderStatusHistories->contains($orderStatusHistory)) {
                        $this->orderStatusHistories[] = $orderStatusHistory;
                        $orderStatusHistory->setOrderShop($this);
                }

                return $this;
        }

        public function removeOrderStatusHistory(OrderStatusHistoryModel $orderStatusHistory): self
        {
                if ($this->orderStatusHistories->contains($orderStatusHistory)) {
                        $this->orderStatusHistories->removeElement($orderStatusHistory);
                        // set the owning side to null (unless already changed)
                        if ($orderStatusHistory->getOrderShop() === $this) {
                                $orderStatusHistory->setOrderShop(null);
                        }
                }

                return $this;
        }

        /**
         * @return Collection|OrderPaymentModel[]
         */
        public function getOrderPayments($meanPayment = null): Collection
        {
                if($meanPayment) {
                        $orderPaymentsReturn = new ArrayCollection() ;

                        foreach($this->orderPayments as $orderPayment) {
                                if($orderPayment->getMeanPayment() == $meanPayment) {
                                        $orderPaymentsReturn[] = $orderPayment ;
                                }
                        }

                        return $orderPaymentsReturn ;
                }

                return $this->orderPayments;
        }

        public function addOrderPayment(OrderPaymentModel $orderPayment): self
        {
                if (!$this->orderPayments->contains($orderPayment)) {
                        $this->orderPayments[] = $orderPayment;
                        $orderPayment->setOrderShop($this);
                }

                return $this;
        }

        public function removeOrderPayment(OrderPaymentModel $orderPayment): self
        {
                if ($this->orderPayments->contains($orderPayment)) {
                        $this->orderPayments->removeElement($orderPayment);
                        // set the owning side to null (unless already changed)
                        if ($orderPayment->getOrderShop() === $this) {
                                $orderPayment->setOrderShop(null);
                        }
                }

                return $this;
        }

        /**
         * @return Collection|OrderProductModel[]
         */
        public function getOrderProducts(): Collection
        {
                return $this->orderProducts;
        }

        public function addOrderProduct(OrderProductModel $orderProduct): self
        {
                if (!$this->orderProducts->contains($orderProduct)) {
                        $this->orderProducts[] = $orderProduct;
                        $orderProduct->setOrderShop($this);
                }

                return $this;
        }

        public function removeOrderProduct(OrderProductModel $orderProduct): self
        {
                if ($this->orderProducts->contains($orderProduct)) {
                        $this->orderProducts->removeElement($orderProduct);
                        // set the owning side to null (unless already changed)
                        if ($orderProduct->getOrderShop() === $this) {
                                $orderProduct->setOrderShop(null);
                        }
                }

                return $this;
        }

        /**
         * @return Collection|CreditHistory[]
         */
        public function getCreditHistories(): Collection
        {
                return $this->creditHistories;
        }

        public function addCreditHistory(CreditHistory $creditHistory): self
        {
                if (!$this->creditHistories->contains($creditHistory)) {
                        $this->creditHistories[] = $creditHistory;
                        $creditHistory->setOrderShop($this);
                }

                return $this;
        }

        public function removeCreditHistory(CreditHistory $creditHistory): self
        {
                if ($this->creditHistories->contains($creditHistory)) {
                        $this->creditHistories->removeElement($creditHistory);
                        // set the owning side to null (unless already changed)
                        if ($creditHistory->getOrderShop() === $this) {
                                $creditHistory->setOrderShop(null);
                        }
                }

                return $this;
        }

        public function getVisitor(): ?Visitor
        {
                return $this->visitor;
        }

        public function setVisitor(?Visitor $visitor): self
        {
                $this->visitor = $visitor;

                return $this;
        }

        public function getDeliveryInfos(): ?string
        {
                return $this->deliveryInfos;
        }

        public function setDeliveryInfos(?string $deliveryInfos): self
        {
                $this->deliveryInfos = $deliveryInfos;

                return $this;
        }


        public function getOrderStatus(): ?OrderStatus
        {
                return $this->orderStatus;
        }

        public function setOrderStatusProtected(?OrderStatus $orderStatus): self
        {
                $this->orderStatus = $orderStatus;

                return $this;
        }


        /**
         * @return Collection|OrderReductionCartModel[]
         */
        public function getOrderReductionCarts(): Collection
        {
                return $this->orderReductionCarts;
        }

        public function addOrderReductionCart(OrderReductionCartModel $orderReductionCart): self
        {
                if (!$this->orderReductionCarts->contains($orderReductionCart)) {
                        $this->orderReductionCarts[] = $orderReductionCart;
                        $orderReductionCart->setOrderShop($this);
                }

                return $this;
        }

        public function removeOrderReductionCart(OrderReductionCartModel $orderReductionCart): self
        {
                if ($this->orderReductionCarts->contains($orderReductionCart)) {
                        $this->orderReductionCarts->removeElement($orderReductionCart);
                        // set the owning side to null (unless already changed)
                        if ($orderReductionCart->getOrderShop() === $this) {
                                $orderReductionCart->setOrderShop(null);
                        }
                }

                return $this;
        }

        /**
         * @return Collection|OrderReductionCartModel[]
         */
        public function getOrderReductionCredits(): Collection
        {
                return $this->orderReductionCredits;
        }

        public function addOrderReductionCredit(OrderReductionCreditModel $orderReductionCredit): self
        {
                if (!$this->orderReductionCredits->contains($orderReductionCredit)) {
                        $this->orderReductionCredits[] = $orderReductionCredit;
                        $orderReductionCredit->setOrderShop($this);
                }

                return $this;
        }

        public function removeOrderReductionCredit(OrderReductionCreditModel $orderReductionCredit): self
        {
                if ($this->orderReductionCredits->contains($orderReductionCredit)) {
                        $this->orderReductionCredits->removeElement($orderReductionCredit);
                        // set the owning side to null (unless already changed)
                        if ($orderReductionCredit->getOrderShop() === $this) {
                                $orderReductionCredit->setOrderShop(null);
                        }
                }

                return $this;
        }

        /**
         * @return Collection|Document[]
         */
        public function getDocuments(): Collection
        {
                return $this->documents;
        }

        public function addDocument(Document $document): self
        {
                if (!$this->documents->contains($document)) {
                        $this->documents[] = $document;
                }

                return $this;
        }

        public function removeDocument(Document $document): self
        {
                if ($this->documents->contains($document)) {
                        $this->documents->removeElement($document);
                }

                return $this;
        }

        public function getDocumentInvoice(): Document
        {
                foreach($this->getDocuments() as $document) {
                        if($document->getType() == Document::TYPE_INVOICE) {
                                return $document ;
                        }
                }

                return false ;
        }

        /**
         * @return Collection|Ticket[]
         */
        public function getTickets(): Collection
        {
                return $this->tickets;
        }

        public function addTicket(Ticket $ticket): self
        {
                if (!$this->tickets->contains($ticket)) {
                        $this->tickets[] = $ticket;
                        $ticket->setOrderShop($this);
                }

                return $this;
        }

        public function removeTicket(Ticket $ticket): self
        {
                if ($this->tickets->contains($ticket)) {
                        $this->tickets->removeElement($ticket);
                        // set the owning side to null (unless already changed)
                        if ($ticket->getOrderShop() === $this) {
                                $ticket->setOrderShop(null);
                        }
                }

                return $this;
        }

        public function isValid() {
                if($this->getOrderStatus() && in_array($this->getOrderStatus()->getAlias(), OrderStatus::$statusAliasAsValid) > 0) {
                        return true ;
                }

                return false ;
        }

        public function isCart() {
                if($this->getOrderStatus() && in_array($this->getOrderStatus()->getAlias(), OrderStatus::$statusAliasAsCart) > 0) {
                        return true ;
                }

                return false ;
        }

        public function getSection(): ?Section
        {
                return $this->section;
        }

        public function setSection(?Section $section): self
        {
                $this->section = $section;

                return $this;
        }

}
